#ifdef GL_ES
precision highp float;
#endif

attribute highp vec4 a_position;
attribute mediump vec2 a_texCoord;

varying mediump vec4 v_colourMultiplier;
varying mediump vec2 v_texCoords1;
varying mediump vec2 v_texCoords2;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp float u_time;
uniform mediump vec4 u_materialEmissive;
uniform mediump vec4 u_materialSpecular;
uniform mediump vec4 u_color4;


void main() {

	// Unpack values from uniforms
	vec2 layerSpeedsMin = u_materialEmissive.rb;
	vec2 layerSpeedsMax = u_materialEmissive.ga;

	float rand = u_materialSpecular.r;
	float pulseTimeMultiplier = u_materialSpecular.g * 25.0;
	float pulseMinimumAlpha = u_materialSpecular.b;
	float combineFactor = u_materialSpecular.a;
	
	float overallAlpha = u_color4.a;


	// Calculate UV offsets for two layers
	vec2 layerSpeeds = mix(layerSpeedsMin, layerSpeedsMax, rand);
	vec2 layerSpeedsNormalised = (layerSpeeds - vec2(0.5)) * vec2(2.0);
	vec2 layerOffsets = (layerSpeedsNormalised * vec2(u_time)) + vec2(rand);
	vec2 layerOffsetsWrapped = fract(layerOffsets);
	
	vec2 layer1uvs = vec2(a_texCoord.x + layerOffsetsWrapped[0], a_texCoord.y);
	vec2 layer2uvs = vec2(a_texCoord.x + layerOffsetsWrapped[1], a_texCoord.y);
	

	// Calculate adjusted time for pulse effect
	float adjustedTime = (u_time + (rand * 6.283185)) * pulseTimeMultiplier;
	float pulsePhase = (sin(adjustedTime) * 0.5) + 0.5;
	float pulseMultiplier = mix(pulseMinimumAlpha, 1.0, pulsePhase);


	// Hand data to frag shader
	v_colourMultiplier = vec4(combineFactor, combineFactor, combineFactor, combineFactor * pulseMultiplier * overallAlpha);
	v_texCoords1 = layer1uvs;
	v_texCoords2 = layer2uvs;

	gl_Position = u_modelViewProjectionMatrix * a_position;

}