// Input per material
uniform lowp vec4 u_color4;
uniform sampler2D u_texture;
uniform lowp vec4 u_materialDiffuse;

// Input per fragment
varying lowp vec2 v_texCoord;
varying mediump float v_fogFactor;
varying lowp vec4 v_color;

// Shader entry point
void main() {

	// Sample texture
	vec4 texColour = texture2D(u_texture, v_texCoord);

	// Are we a blinking thing?
	float dontBlink = v_color.a;

	// Grab fog colour from uniform and blend it with black if we're a blinking thing
	vec3 fogColour = mix(vec3(0.0, 0.0, 0.0), u_materialDiffuse.rgb, dontBlink);

	// Calculate fog factor (should really be in vert shader)
	float fogFactor = 1.0 - v_fogFactor;
	fogFactor = 1.0 - (fogFactor * fogFactor * fogFactor);
	fogFactor = clamp(fogFactor, 0.0, u_materialDiffuse.a);

	// Create a final colour by lerping between the texture colour
	// and the fog colour, by the fog factor
	vec3 finalColour = mix(texColour.rgb, fogColour, fogFactor);

	gl_FragColor = vec4(finalColour, texColour.a) * vec4(v_color.rgb, 1.0) * u_color4;
}